classdef Model < falcon.core.Model
    % FALCON.PAREST.CORE.MODEL parameter estimation specific model
    %  
    %  	<Description>
    %  	this class wraps the falcon.core.Model class for use in parameter
    %  	estimation.
    %   
    %   TODO:
    %   * avoid checking dimensions every time the simulation is called
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % flag, indicating if sensitivities are to be updated or not
        UseSensitivities
        % link back to experiment
        Experiment
        % number of states
        nX
        % number of outputs
        nY
        % number of controls
        nU
        % number of process noise inputs
        nW
        % number of parameters
        nP
        % number of samples
        N
        % state sensitivities w.r.t. initial condition
        dx_dx0
        % state sensitivities w.r.t. inputs
        dx_du
        % state sensitivities w.r.t. parameters
        dx_dp
        % output sensitivities w.r.t. initial conditions
        dy_dx0
        % output sensitivities w.r.t. inptus
        dy_du
        % output sensitivities w.r.t. parameters
        dy_dp
        % name of the model (and mex files)
        ModelName
        % The phase this falcon.core.Model belongs to
        Phase
        % The grid for the state dot values
        StateDotGrid
        % The grid for the model outputs
        ModelOutputGrid
        % The grid for the simulated state dot values
        SimulatedStateDotGrid
        % The grid for the simulated model outputs
        SimulatedOutputGrid
        % Model calculate the dynamics either by using Model.Simulate()
        % for shooting methods or Model.Evaluate() for Collocation
        % methods
        ModelHandle
        % The struct holding all information about the model function.
        ModelInfoStruct
        % The parameters used in the simulation of the model
        ModelParameters
        % The constants used for the simulation of the model
        ModelConstants
        % The indices of the states to be integrated using the refined
        % multiple shooting grid.
        ShootingIndices
        % The indices of the states to be integrated using the coarse
        % collocation grid.
        SlowIndices
        % Flag that determines whether we have a real, physical output
        % of the system
        real_out_avail
        % PathFunction object generated for the output path constraints
        PathConstraintFunction
        % Specifies whether the Model has Outputs
        hasOutputs
    end

    methods
        function filter(obj, x0, z, Q, R, P_xx_0, OutputsConsidered, SmootherFlag, E)
        % FILTER do Kalman Filtering / RTS Smoothing
        %  
        %  	<Syntax>
        %  	filter( obj, x0, z, Q, R, P_xx_0, OutputsConsidered, SmootherFlag, E )
        %  
        %  	<Description>
        %  	call mex file created by falcon.parest.SimulationModelBuilder
        %  	in order to perform forward integration based on initial
        %  	condition x0.
        %  	Results are stored in the respective grids. Sort input data
        %  	such, that simulation can be performed in parralel
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %  	>x0: initial condition
        %  	>z: cell of measurement data arrays
        %  	>Q: 3D matrix of process noise covariance matrices
        %  	>R: 3D matrix of measurement noise covariance matrices
        %  	>P_xx_0: 3D matrix of initial state covariance matrices
        %  	>OutputsConsidered: logical array, indicating which outputs
        %  	to use during filtering
        %  	>SmootherFlag: flag indicating if smoother is to be used
        %  	>E: process noise feedthrough matrix
        %  
        %  	<ChangeLog>
        %  	2017-02-11 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function updateOutputsAndJacobians(obj)
        % UPDATEOUTPUTSANDJACOBIANS update for new model parameters
        %  
        %  	<Syntax>
        %  	updateOutputsAndJacobians( obj )
        %  
        %  	<Description>
        %  	update jacobians and model outputs for a new set of model
        %  	parameters; keep the state values constant at their initial
        %  	guess!
        %  
        %   This function is used in the process of improving an
        %   initial parameter guess.
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function InitialStateSensitivityIntegration(obj)
        % INITIALSTATESENSITIVITYINTEGRATION integrate sensitivity
        %  
        %  	<Syntax>
        %  	InitialStateSensitivityIntegration( obj )
        %  
        %  	<Description>
        %  	integrate sensitivities based on state initial guess (not
        %  	on integrated states or states from collocation problem!)
        %  	to improve upon initial parameter guess
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function simulate(obj, x0, varargin)
        % SIMULATE do forward simulation
        %  
        %  	<Syntax>
        %  	simulate( obj, x0 )
        %  
        %  	<Description>
        %  	call mex file created by falcon.parest.SimulationModelBuilder
        %  	in order to perform forward integration based on initial
        %  	condition x0.
        %  	Results are stored in the respective grids. Sort input data
        %  	such, that simulation can be performed in parralel
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %  	>x0: initial condition
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-02-11 - Ch. Goettlicher: split into sub-functions
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2017-06-26 - Ch. Goettlicher: adjust to be compatible with
        %   Simulink wrapper
        end

        function integrateSensitivitiesExplicitTrapezoidal(obj)
        % INTEGRATESENSITIVITIESEXPLICITTRAPEZOIDAL sensitivity
        %   integration using trapezoidal rule
        %  
        %  	<Syntax>
        %  	integrateSensitivitiesExplicitTrapezoidal( obj )
        %  
        %  	<Description>
        %  	use an explicit formulation of the trapezoidal integration
        %  	scheme to integrate the sensitivity equations after a
        %  	collocation based estimation had been performed. Store them
        %  	in the respective grids (state and output). The integration
        %  	is based on
        %  	
        %  	S_i = S_{i-1} + \Delta t / 2 * (\dot{S}_{i-1} + \dot{S}_i)
        %  	\dot{S}_i = (df/dx)_i * S_i + (df/dp)_i = A_i*S_i + F_i
        %  	==> S_i = (I - \Delta t/2 * A_i)^-1 *
        %  	(S_{i-1} + \Delta t/2 * (A_{i-1}*S_{i-1} + F_i + F_{i-1})
        %  	(dy/dp)_i = (dg/dx)_i*S_i + (dg/dp)_i
        %  
        %  	<Inputs>
        %  	>obj: the Model object
        %  
        %  	<ChangeLog>
        %   2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2017-06-26 - Ch. Goettlicher: change to compiled
        %   integration routine
        end

        function obj = Model(Problem, Phase, ModelHandle)
        % MODEL constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = Model( obj, Phase, ModelHandle )
        %  
        %  	<Description>
        %  	create the falcon.parest.Model object by passing all
        %  	arguments to the superclass constructor.
        %  
        %  	<Inputs>
        %  	>Problem: the problem to which this model is associated
        %  	>Phase: the phase to which this model is associated
        %  	>ModelHandle: the model handle of this model (created using
        %  	the simulation model builder)
        %  
        %  	<Outputs>
        %  	>obj: the newly created Model object
        %  
        %  	<ChangeLog>
        %  	2017-01-24 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2017-06-26 - Ch. Goettlicher: adjust for FEM problem and to
        %   be compatible with Simulink wrapper
        end

        function strc = ToStruct(obj, varargin)
        % Create a struct of the falcon.core.Model-object.
        %  
        % <Syntax>
        % strc = obj.ToStruct()
        %  
        % <Description>
        % This metod creates a struct of the falcon.core.Model-object
        % including all the necessary information of the Model.
        %  
        % <Inputs>
        % >obj: falcon.core.Model-object to be transformed in a struct.
        %  
        % <Outputs>
        % >strc: struct containing the inherent properties of the falcon.core.Model-object
        %  
        % <NameValue>
        % >DebugData: Setting this option to true enables debug data in
        %   the ToStruct method.
        %  
        % <Keywords>
        % Debugging!Model
        end

        function setModelOutputs(obj, Outputs)
        % Set the given falcon.Outputs as the outputs of the model.
        %  
        % <Syntax>
        % obj.setModelOutputs(Outputs)
        %  
        % <Description>
        % Sets the given array of falcon.Output objects as the outputs additional
        % outputs for the model. In case there are finite limits defined for the
        % outputs, the outputs are automatically limited to the respective values.
        %  
        % <Inputs>
        % > Outputs: An array of falcon.Output objects used to store the outputs of
        % the model.
        %  
        % <Keywords>
        % Model!Outputs
        end

        function overwriteConstants(obj, ConstantsCell, ConstantsIdx)
        % Overwrites the given numbers in the list of constants used
        % in this simulation model.
        %  
        % <Syntax>
        % obj.overwriteConstants(Constant,...)
        %  
        % <Description>
        % Overwrites the given cell of constants in the list of
        % constants relevant for this simulation model.
        %  
        % <Inputs>
        % > ConstantsCell: A cell of constant numbers used in this
        % PathFunction.
        % > ConstantsIdx: An array of the size of constants cell
        % containing the indices of the constants that should be
        % overwritten.
        %  
        % <Keywords>
        % Model!Overwrite Constants
        end

        function addModelConstants(obj, varargin)
        % Add the given numbers to the list of constants used in the simulation
        % model.
        %  
        % <Syntax>
        % obj.addModelConstants(Constant1, Constant2, ..)
        %  
        % <Description>
        % Adds the given array of constants to the list of constants relevant for
        % the simulation model.
        %  
        % <Inputs>
        % > Constant1: An array of constant numbers used in this
        % PathFunctionfor the first constant.
        % > Constant2: An array of constant numbers used in this
        % PathFunctionfor the second constant.
        % > INFO: Note that the third matrix dimension for constants
        % with MultipleTimeEvaluation set to true contains the
        % evolution over the time steps, i.e., constants of size [n,m]
        % for each discretization step in the phase.
        %  
        % <Keywords>
        % Model!Constants
        end

        function setModelParameters(obj, Parameters)
        % Set the given parameters as the parameters used in the simulation model.
        %  
        % <Syntax>
        % obj.setModelParameters(Parameters)
        %  
        % <Description>
        % Sets the given array of parameters as the parameters relevant for the
        % simulation model. All parameters given here will be used as the thrid
        % input to the simulation model dynamics.
        %  
        % <Inputs>
        % > Parameters: An array of falcon.Parameter objects used in the simulation of
        % the dynamic model.
        %  
        % <Keywords>
        % Model!Parameters
        end

    end

end