classdef ExperimentField < falcon.core.Handle
    % FALCON.PAREST.PLOT.EXPERIMENTFIELD
    %  
    %  	<Description>
    %  	an "ExperimentField" is one section of a subplot, where the data
    %  	from one output is plotted. Thus, an experiment figure can be
    %  	thought of as a matrix of "ExperimentFields" where the rows are
    %  	related to different output, and the coloumns are related to
    %  	different experiments
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % handle of the axes object that contains this field's data
        SubplotHandle
    end

    methods
        function retVal = isExistent(obj)
        % ISEXISTENT check if measurement data is present
        %  
        %  	<Syntax>
        %  	[ retVal ] = isExistent( obj )
        %  
        %  	<Description>
        %  	check if measurement data has been plotted by investigation
        %  	the ExperimentHandle property, which is of type
        %  	matlab.graphics.GraphicsPlaceholder if no plot exists yet.
        %  	Additionally, check that the line plot is valid.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  
        %  	<Outputs>
        %  	>retVal: array of logicals (with same size as object-array)
        %  	indicating if measurement data has been plotted
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function retVal = hasModelPlot(obj)
        % HASMODELPLOT check if model data has been plotted
        %  
        %  	<Syntax>
        %  	[ retVal ] = hasModelPlot( obj )
        %  
        %  	<Description>
        %  	check if model data has been plotted by investigation the
        %  	ModelHandle property, which is of type
        %  	matlab.graphics.GraphicsPlaceholder if no plot exists yet.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  
        %  	<Outputs>
        %  	>retVal: array of logicals (with same size as object-array)
        %  	indicating if model data has been plotted
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setPlotDelta_t(obj, PlotDelta_t)
        % SETPLOTDELTA_T set time offest
        %  
        %  	<Syntax>
        %  	setPlotDelta_t( obj, PlotDelta_t )
        %  
        %  	<Description>
        %  	set offset in time, which serves to plot all fields next to
        %  	each other
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  	>PlotDelta_t: time shift to arrive at a plot with all
        %  	experiments next to each other
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function plotBar(obj)
        % PLOTBAR plot bars at the end of the fields to have a visual
        %   separation
        %  
        %  	<Syntax>
        %  	plotBar( obj )
        %  
        %  	<Description>
        %  	at the end of the current field, plot a vertical bar in order
        %  	to separate it from the following field
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function legend(obj, LegendEntries, varargin)
        % LEGEND add a legend
        %  
        %  	<Syntax>
        %  	legend( obj, LegendEntries, varargin )
        %  
        %  	<Description>
        %  	add a legend the to subplot associated with this field. All
        %  	variable arguments are passed directly to the standard matlab
        %  	'legend' function. Only two legend entries (for measured and
        %  	model output) should be provided.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  	>LegendEntries: the two legend entries to be associated with
        %  	the experiment and model output
        %  	>varargin: list of arguments to be passed to underlying
        %  	legend function
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function updateExperimentData(obj, y)
        % falcon.parest.plot.ExperimentField/updateExperimentData is a function.
        %   updateExperimentData(obj, y)
        end

        function updateModelOutput(obj, y)
        % UPDATEMODELOUTPUT update model output data
        %  
        %  	<Syntax>
        %  	updateModelOutput( obj, y )
        %  
        %  	<Description>
        %  	update the model output data by adjusting the corresponding
        %  	.YData property of the ModelHandle property. This is a lot
        %  	faster than plotting the data anew, and is used during
        %  	iteration.
        %  	before an update, it is checked if the corresponding
        %  	measurement plot exists, and if there is model data to be
        %  	updated, i.e. if the method plotModelData has been called for
        %  	this object in the past.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  	>y: new model output data
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function plotModelData(obj, t, y, PlotCfg)
        % PLOTMODELDATA plot model output data
        %  
        %  	<Syntax>
        %  	plotModelData( obj, t, y, PlotCfg )
        %  
        %  	<Description>
        %  	can be called for an array of fields, then in every field
        %  	(determined by the SubplotHandle and respective PlotDelta_t
        %  	property), the model output data is plotted. However, this is
        %  	only done, if the corresponding measurement plot exists.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  	>t: time vector
        %  	>y: model output data
        %  	>PlotCfg: plot config struct with fields 'Color',
        %  	'LineSyle', 'LineWidth'
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: adjust for two model plots in
        %   TIKZ export
        end

        function adjustYLim(obj, YLim, varargin)
        % ADJUSTYLIM adjust y-limits to display all relevant data
        %  
        %  	<Syntax>
        %  	adjustYLim( obj )
        %  
        %  	<Description>
        %  	look for all line handles (ignoring separation bars) and set
        %  	the plot y-limits such, that all data is displayed.
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %  
        %  	<ChangeLog>
        %  	2017-02-20 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function plotBounds(obj, UpperBound, LowerBound)
        % falcon.parest.plot.ExperimentField/plotBounds is a function.
        %   plotBounds(obj, UpperBound, LowerBound)
        end

        function plotExperimentData(obj, Parent, t, data, PlotCfg, UserData)
        % PLOTEXPERIMENTDATA plot experiment data in this field
        %  
        %  	<Syntax>
        %  	plotExperimentData( obj, Parent, t, data, PlotCfg, UserData )
        %  
        %  	<Description>
        %  	plot experiment data (controls or measurements) in the
        %  	current field. the data is time-shifted by the PlotDelta_t
        %  	property, in order to ensure, that all experiments are
        %  	plotted next to each other. The appearance of the plot is
        %  	configure via the respective PlotCfg_xxxx structs of the
        %  	falcon.parest.plot.Experiment class
        %  
        %  	<Inputs>
        %  	>obj: the ExperimentField object
        %   >Parent: parent subplot 
        %  	>t: time vector
        %  	>data: data vector (control or measurement)
        %  	>PlotCfg: plot config struct with fields 'Color',
        %  	'LineSyle', 'LineWidth'
        %  	>UserData:
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-03-08 - Ch. Goettlicher: bugfix for empty measurement
        %                data
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %  	2018-11-02 - Ch. Goettlicher: added parent subplot argument
        end

        function setSignalName(obj, Name)
        % falcon.parest.plot.ExperimentField/setSignalName is a function.
        %   setSignalName(obj, Name)
        end

        function plotUncertaintyData(obj, t, val, std, PlotCfg, UserData)
        % falcon.parest.plot.ExperimentField/plotUncertaintyData is a function.
        %   plotUncertaintyData(obj, t, val, std, PlotCfg, UserData)
        end

        function obj = ExperimentField()
        % EXPERIMENTFIELD constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = ExperimentField( obj )
        %  
        %  	<Description>
        %  	the constructor for an empty
        %  	falcon.parest.plot.ExperimentField object
        %  
        %  	<Outputs>
        %  	>obj: the newly created ExperimentField object
        %  
        %  	<ChangeLog>
        %  	2017-02-05 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end