classdef (Abstract) Parameter < falcon.core.Handle
    % FALCON.PAREST.PLOT.PARAMETER 
    %  
    %  	<Description>
    %  	plotting functionalities for parameter estimation parameters
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % base color for std-deviation and correlation plots
        BaseColor
        % parameter labels for figures
        Label
        % abstract property for plotting standard deviations
        StandardDeviation
        % abstract property for plotting relative standard deviations
        RelStandardDeviation
    end

    methods
        function setLabels(obj, Labels)
        % SETLABELS set parameter labels
        %  
        %   <Syntax>
        %   obj.setLabels( Labels )
        %  
        %   <Description>
        %   set falcon.parest.Parameter.Label value by passing a cell
        %   array of strings of proper size to this function. If this
        %   method is called for an array of parameters, the labels
        %   will be assigned to parameters accordingly.
        %   <Inputs>
        %   >Labels: cell array of strings containing the label of the
        %   parameters
        %  
        %  	<ChangeLog>
        %   2017-12-19 - X. Fank: initial design
        %  	2017-01-28 - Ch. Goettlicher: moved label things
        end

        function plotCorrelationMatrix(obj)
        % PLOTCORRELATIONMATRIX 3D bar plot of correlation matrix
        %  
        %  	<Syntax>
        %  	plotCorrelationMatrix( obj )
        %  
        %  	<Description>
        %  	creates a 3D bar plot of the absolute value of the
        %  	correlation matrix. The larger the values, the more red the
        %  	bars become.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: sync with Goettlicher TIKZ
        %   exporter
        end

        function plotBar(obj, varargin)
        % PLOTBAR make bar plot with parameter values
        %  
        %  	<Syntax>
        %  	plotBar( obj )
        %  	plotBar( obj, Name, Value )
        %  
        %  	<Description>
        %  	create a bar plot with the parameter values, standard
        %  	deviations, and possibly initial and true values
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >plotInitialValues: integer to indicate if initial values
        %   are to be plotted, too (1), or if the initial values after
        %   refinement are to be plotted (2).
        %   >trueValues: array of true parameter values for simulation
        %   examples
        %   >nSigma: integer to indicate how many standard deviations
        %   shall be used for the errorbar plot
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function plotRelativeStandardDeviation(obj, varargin)
        % PLOTRELATIVESTANDARDDEVIATION bar plot of relative standard
        %   deviation
        %  
        %  	<Syntax>
        %  	plotRelativeStandardDeviation( obj )
        %  	plotRelativeStandardDeviation( obj, InitialValueFlag )
        %  
        %  	<Description>
        %  	creates a horizontal bar plot of the relative standard
        %  	deviations in the current axis
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >InitialValueFlag: flag to indicate, if initial value is to
        %   be plotted alongside
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %  	2018-01-19 - Ch. Goettlicher: added initial value
        end

        function plotParameterEvolution(obj, varargin)
        % PLOTPARAMETEREVOLUTION short description
        %  
        %  	<Syntax>
        %  	plotParameterEvolution( obj )
        %  	plotParameterEvolution( obj, 'Name', Value )
        %  
        %  	<Description>
        %  	plot the evolution of the parameter values during the
        %  	iteration. the last value and its standard deviation are
        %  	highlighted with a red line and an error bar.
        %   if any improvement algorithm on the initial guess was used,
        %   these improvements will be highlightes with circles
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<NameValue>
        %  	>SubplotConfig: two element vector containing the number of
        %  	vertical and horizontal subplots to create
        %  	>Normalized: logical indicating if the true parameter value
        %  	or its normalized history is to be plotted
        %   >showDifference: show difference to initial value in
        %   percent to illustrate the change during the optimization
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: added initial improvement
        %   marker; adjusted parameter labels
        end

        function obj = Parameter()
        % FALCON.PAREST.PLOT.PARAMETER 
        %  
        %  	<Description>
        %  	plotting functionalities for parameter estimation parameters
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end