classdef Parameter < falcon.parest.plot.Parameter & falcon.Parameter & falcon.parest.core.PEVC & falcon.core.HasToStruct & falcon.parest.core.hasCovariance
    % FALCON.PAREST.PARAMETER parameter variables for parameter estimation
    %  
    %  	<Description>
    %  	wrapper for parameter variables for parameter estimation. 
    %  
    %   some additional data is computed and stored, when compared to the
    %   standard falcon.Parameter class, e.g. covariances, and the
    %   evolution during the estimation process.
    %  
    %  	<ChangeLog>
    %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
    %   2018-06-26 - Ch. Goettlicher: moved label stuff to
    %   falcon.parest.plot.Parameter

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % Flag to indicate if Bayesian estimation is to be used for this
        % parameter
        Bayesian
        % falcon.parest.Parameter/BayesianValue is a property.
        BayesianValue
        % base color for std-deviation and correlation plots
        BaseColor
        % parameter labels for figures
        Label
        % The current value of this parameter
        Value
        % Index the index of this parameter in the z-Vector
        Index
        % Scaling Parameters initialized with 1. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Scaling
        % Offset parameter which is initialized with 0. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Offset
        % Name of object.
        Name
        % Lower bound of the falcon.core.OVC value. It is initialized with
        % minus infinity. Scaling and Offset are applied to the
        % LowerBound as well.
        LowerBound
        % Upper bound of the falcon.core.OVC value. It is initialized with
        % plus infinity. Scaling and Offset are applied to the
        % UpperBound as well.
        UpperBound
        % Whether this value is fixed or not.
        isFixed
        % standard deviation of this parameter
        StandardDeviation
        % relativ standard deviation, i.e. sigma/value
        RelStandardDeviation
    end

    methods
        function Covariance = getBayesianCovariance(obj)
        % falcon.parest.Parameter/getBayesianCovariance is a function.
        %   Covariance = getBayesianCovariance(obj)
        end

        function storeValueAsBayesian(obj)
        % falcon.parest.Parameter/storeValueAsBayesian is a function.
        %   storeValueAsBayesian(obj)
        end

        function setBayesian(obj, isBayesian)
        % falcon.parest.Parameter/setBayesian is a function.
        %   setBayesian(obj, isBayesian)
        end

        function setFixed(obj, isFixed)
        % SETFIXED set parameters fixed
        %  
        %  	<Syntax>
        %  	setFixed( obj, isFixed )
        %  
        %  	<Description>
        %  	set falcon.parest.Parameter.isFixed value. if a scalar is
        %  	input to this function, all elements of the parameter array
        %  	are assigned with this value. If a logical array of proper
        %  	size is input, the parameters are set fixed accordingly.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>isFixed: logical scalar or array of proper size. new value
        %  	of falcon.parest.Parameter.isFixed property
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-22 - Ch. Goettlicher: bug fix for ND obj arrays
        end

        function setValue(obj, Value)
        % SETVALUE set new value without storing
        %  
        %  	<Syntax>
        %  	setValue( obj, Value )
        %  
        %  	<Description>
        %  	set new value for this parameter array
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: new parameter value
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setValueByName(obj, Value, Name)
        % SETVALUEBYNAME set a parameter value by name
        %  
        %  	<Syntax>
        %  	setValueByName(obj, Value, Name)
        %  
        %  	<Description>
        %  	set the provided Values for the parameters provided in the
        %  	cell array Name
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: the new parameter values
        %   >Name: cell array of parameter names
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function updateValue(obj, Value)
        % UPDATEVALUE set new value and store
        %  
        %  	<Syntax>
        %  	updateValue( obj, Value )
        %  
        %  	<Description>
        %  	change the value of the parameter and store it to the
        %  	falcon.parest.Parameter.Evolution property
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: the new parameter value
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function [s, cov] = ToStruct(obj)
        % TOSTRUCT store parameter information in struct
        %  
        %  	<Syntax>
        %  	[ s, cov ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	store all information of this parameter object in a struct.
        %  	Can be used to store the parameter data permanently, even if
        %  	the class definition changes (which would make storing the
        %  	complete object of a former version useless)
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>s: the parameter's information in a struct.
        %  	>cov: the parameter's covariance, for the non-fixed
        %  	parameter
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-02-07 - Ch. Goettlicher: bugfix for fixed parameters
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function ToFile(obj, FullFileName)
        % TOFILE store parameter data to file
        %  
        %  	<Syntax>
        %  	ToFile( obj, FullFileName )
        %  
        %  	<Description>
        %  	convert parameter data to struct and store this struct to a
        %  	file. the conversion to a struct makes the stored file
        %  	independent of the version of the class in use.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>FullFileName: full path and filename of the file to store
        %  	the parameter data in
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function loadValuesFromFile(obj, FullFileName)
        % LOADVALUESFROMFILE load values from file
        %  
        %  	<Syntax>
        %  	loadValuesFromFile( obj, FullFileName )
        %  
        %  	<Description>
        %  	load the parameter values from a file, where only the
        %  	values that can be found in the file are set. The values
        %  	are also set for 'fixed' parameters!!! To avoid this, call
        %        p(~[p.isFixed]).loadValuesFromFile(FullFileName)
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>FullFileName: filename to load data from
        %  
        %  	<ChangeLog>
        %  	2017-01-16 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = Parameter(ParamName, varargin)
        % PARAMETER constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = Parameter( ParamName, varargin )
        %  
        %  	<Description>
        %  	object describing the parameter characteristics, e.g. name,
        %  	value, boundaries.
        %  	After the estimation, statistical characteristics (cross
        %  	variances with itself and other parameters) can be computed
        %  	and stored along with the value of the parameter.
        %  
        %  	<Inputs>
        %  	>ParamName: parameter name; should correspond to the model
        %  	parameter names
        %  
        %  	<NameValue>
        %  	>Value: The current (initial) value of this parameter to be
        %  	passed to the underlying falcon.Parameter object
        %  	>LowerBound: the lower bound of this parameter to be
        %  	passed to the underlying falcon.Parameter object
        %  	>UpperBound: the upper bound of this parameter to be
        %  	passed to the underlying falcon.Parameter object
        %  	>Scaling: the scaling of this parameter to be
        %  	passed to the underlying falcon.Parameter object (currently
        %  	not used for estimation using the ML solver)
        %  	>Offset: the offset of this parameter to be
        %  	passed to the underlying falcon.Parameter object (currently
        %  	not used for estimation using the ML solver)
        %  	>Fixed: flag indicating if this parameter is to be kept
        %  	fixed. Value is passed to the underlying falcon.Parameter
        %  	object
        %  
        %  	<Outputs>
        %  	>obj: the newly created Parameter object
        %  
        %  	<ChangeLog>
        %  	2017-01-27 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = FromFile(FullFileName)
        % FROMFILE load parameter from file
        %  
        %  	<Syntax>
        %  	[ obj ] = falcon.parest.Parameter.FromFile( FullFileName )
        %  
        %  	<Description>
        %  	create a new falcon.parest.Parameter object from a struct
        %  	stored in a file
        %  
        %  	<Inputs>
        %  	>FullFileName: full path and file name to the file
        %  	containing the Parameter data
        %  
        %  	<Outputs>
        %  	>obj: the newly created Parameter object
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setLabels(obj, Labels)
        % SETLABELS set parameter labels
        %  
        %   <Syntax>
        %   obj.setLabels( Labels )
        %  
        %   <Description>
        %   set falcon.parest.Parameter.Label value by passing a cell
        %   array of strings of proper size to this function. If this
        %   method is called for an array of parameters, the labels
        %   will be assigned to parameters accordingly.
        %   <Inputs>
        %   >Labels: cell array of strings containing the label of the
        %   parameters
        %  
        %  	<ChangeLog>
        %   2017-12-19 - X. Fank: initial design
        %  	2017-01-28 - Ch. Goettlicher: moved label things
        end

        function plotCorrelationMatrix(obj)
        % PLOTCORRELATIONMATRIX 3D bar plot of correlation matrix
        %  
        %  	<Syntax>
        %  	plotCorrelationMatrix( obj )
        %  
        %  	<Description>
        %  	creates a 3D bar plot of the absolute value of the
        %  	correlation matrix. The larger the values, the more red the
        %  	bars become.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: sync with Goettlicher TIKZ
        %   exporter
        end

        function plotBar(obj, varargin)
        % PLOTBAR make bar plot with parameter values
        %  
        %  	<Syntax>
        %  	plotBar( obj )
        %  	plotBar( obj, Name, Value )
        %  
        %  	<Description>
        %  	create a bar plot with the parameter values, standard
        %  	deviations, and possibly initial and true values
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >plotInitialValues: integer to indicate if initial values
        %   are to be plotted, too (1), or if the initial values after
        %   refinement are to be plotted (2).
        %   >trueValues: array of true parameter values for simulation
        %   examples
        %   >nSigma: integer to indicate how many standard deviations
        %   shall be used for the errorbar plot
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function plotRelativeStandardDeviation(obj, varargin)
        % PLOTRELATIVESTANDARDDEVIATION bar plot of relative standard
        %   deviation
        %  
        %  	<Syntax>
        %  	plotRelativeStandardDeviation( obj )
        %  	plotRelativeStandardDeviation( obj, InitialValueFlag )
        %  
        %  	<Description>
        %  	creates a horizontal bar plot of the relative standard
        %  	deviations in the current axis
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >InitialValueFlag: flag to indicate, if initial value is to
        %   be plotted alongside
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %  	2018-01-19 - Ch. Goettlicher: added initial value
        end

        function plotParameterEvolution(obj, varargin)
        % PLOTPARAMETEREVOLUTION short description
        %  
        %  	<Syntax>
        %  	plotParameterEvolution( obj )
        %  	plotParameterEvolution( obj, 'Name', Value )
        %  
        %  	<Description>
        %  	plot the evolution of the parameter values during the
        %  	iteration. the last value and its standard deviation are
        %  	highlighted with a red line and an error bar.
        %   if any improvement algorithm on the initial guess was used,
        %   these improvements will be highlightes with circles
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<NameValue>
        %  	>SubplotConfig: two element vector containing the number of
        %  	vertical and horizontal subplots to create
        %  	>Normalized: logical indicating if the true parameter value
        %  	or its normalized history is to be plotted
        %   >showDifference: show difference to initial value in
        %   percent to illustrate the change during the optimization
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: added initial improvement
        %   marker; adjusted parameter labels
        end

        function self = setValueAndRange(self, value, range)
        % falcon.Parameter/setValueAndRange is a function.
        %   [self] = setValueAndRange(self, value, range)
        end

        function self = setValueAndBounds(self, value, lowerBound, upperBound)
        % falcon.Parameter/setValueAndBounds is a function.
        %   [self] = setValueAndBounds(self, value, lowerBound, upperBound)
        end

        function obj = setOffset(obj, offset)
        % Set the offset of this object.
        %  
        % <Syntax>
        % obj.setOffset(offset)
        %  
        % <Description>
        % Set the offset of this object. Please note
        % that the input must be a real, scalar value.
        %  
        % <Inputs>
        % >offset: Offset of this object.
        %  
        % <Keywords>
        % OVC!Offset
        end

        function obj = setScaling(obj, scaling)
        % Set the scaling of this object.
        %  
        % <Syntax>
        % obj.setScaling(scaling)
        %  
        % <Description>
        % The input must be a real positive scalar value and should
        % scale the value of this object to a range between -1 and 1.
        %  
        % <Inputs>
        % >scaling: Scaling of the this object.
        %  
        % <Keywords>
        % OVC!Scaling
        end

        function element = findFirst(self, pattern)
        % Find the first element matching a regex.
        %  
        % <Syntax>
        % element = array.findFirst(pattern)
        %  
        % <Description>
        % Extract the first element from the array whose Name attribute
        % matches the given pattern, as determined by the regexp function.
        % If no element matches, an exception is thrown.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > element: the first matching element
        end

        function elements = find(self, pattern)
        % Find elements matching a regex.
        %  
        % <Syntax>
        % elements = array.find(pattern)
        %  
        % <Description>
        % Extract object array elements whose Name attributes match the
        % given pattern, as determined by the regexp function.  The
        % returned array is empty if no element matches.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > elements: the entries of the object array whose names match
        end

        function bool = ne(varargin)
        % ~= (NE)   Not equal relation for handles.
        %   Handles are equal if they are handles for the same object and are 
        %   unequal otherwise.
        %  
        %   H1 ~= H2 performs element-wise comparisons between handle arrays H1 
        %   and H2.  H1 and H2 must be of the same dimensions unless one is a 
        %   scalar.  The result is a logical array of the same dimensions, where 
        %   each element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = NE(H1, H2) stores the result in a logical array of the same
        %   dimensions.
        end

        function bool = eq(varargin)
        % == (EQ)   Test handle equality.
        %   Handles are equal if they are handles for the same object.
        %  
        %   H1 == H2 performs element-wise comparisons between handle arrays H1 and
        %   H2.  H1 and H2 must be of the same dimensions unless one is a scalar.
        %   The result is a logical array of the same dimensions, where each
        %   element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = EQ(H1, H2) stores the result in a logical array of the same 
        %   dimensions.
        end

        function map = byNameUnique(self)
        % Get a struct for name based access to the object array, allowing
        % only unique names.
        %  
        % <Syntax>
        % [ map ] = obj.byNameUnique()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  Throws
        % an error in case there are any duplicate names.
        %  
        % <Keywords>
        % OVC!byName!Unique
        end

        function map = byName(self)
        % Get a struct for name based access to the object array.
        %  
        % <Syntax>
        % [ map ] = obj.byName()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  In case
        % there are duplicate names, the output struct contains object
        % arrays.
        %  
        % <Keywords>
        % OVC!byName
        end

        function self = relaxRange(self, range)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = relaxBounds(self, lowerBound, upperBound)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = tightenRange(self, range)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = tightenBounds(self, lowerBound, upperBound)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = setRange(self, range)
        % Set the lower and upper bound using a vector argument.
        %  
        % <Syntax>
        % obj.setRange(range)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > range: a two-element double vector
        end

        function self = setBounds(self, lowerBound, upperBound)
        % Set the lower and upper bound.
        %  
        % <Syntax>
        % obj.setBounds(lowerBound, upperBound)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function obj = setUpperBound(obj, upperBound)
        % Set the upper bound of this object.
        %  
        % <Syntax>
        % obj.setUpperBound(upperBound)
        %  
        % <Description>
        % Set the upper bound of this object. The input must be a real, scalar
        % value and bigger than the lower bound.
        %  
        % <Inputs>
        % >upperBound: Upper bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Upper
        end

        function obj = setLowerBound(obj, lowerBound)
        % Set the lower bound of this object.
        %  
        % <Syntax>
        % obj.setLowerBound(lowerBound)
        %  
        % <Description>
        % Set the lower bound of this object. The input must be a real, scalar
        % scalar and smaller than the upper bound.
        %  
        % <Inputs>
        % >lowerBound: Lower bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Lower
        end

        function Correlation = getCorrelation(obj)
        % GETCORRELATION get correlation matrix for an array of
        %   parameters
        %  
        %  	<Syntax>
        %  	[ Correlation ] = getCorrelation( obj )
        %  
        %  	<Description>
        %  	get the correlation matrix for an array of parameters. It is
        %  	computed from falcon.parest.Parameter.getCovariance and the
        %  	corresponding standard deviations
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>Correlation: correlation matrix
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setVariances(obj, CrossVariance, varargin)
        % SETVARIANCES set parameter variances
        %  
        %  	<Syntax>
        %  	setVariances( obj, CrossVariance, CrossVarianceParameters )
        %  
        %  	<Description>
        %  	store the cross variances with other parameters. Both the
        %  	cross variances and the respective parameters are stored.
        %  	However these two arrays are sorted such, that the first
        %  	entry always is the parameter itself, i.e. the first
        %  	covariance is the parameter variance.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>CrossVariance: double array of cross variance parameters
        %  	>CrossVarianceParameters: Parameter objects which are
        %  	associated with the corresponding entries in the
        %  	CrossVariance input
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function Covariance = getCovariance(obj)
        % GETCOVARIANCE get covariance matrix of a parameter array
        %  
        %  	<Syntax>
        %  	[ Covariance ] = getCovariance( obj )
        %  
        %  	<Description>
        %  	obtain the covariance matrix for an array of parameters. to
        %  	do so, the Cross-Variance-Parameters are compared to the
        %  	parameter array (obj), and the covariance matrix is built up
        %  	from the cross variances
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>Covariance: Covariance matrix for this array of parameters
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end