classdef Result < falcon.core.Handle & falcon.core.HasToStruct
    % FALCON.TRIM.LIN.RESULT holds linearization results
    %  
    %  	<Description>
    %  	class to assemble all results from the linearizer
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % grid containint linearization results
        Results
        % state names from state-space syste
        StateName
        % control names from state-space system
        ControlName
        % output names from state-space system
        OutputName
    end

    methods
        function toFile(obj, FileName)
        % TOFILE store data in file
        %  
        %  	<Syntax>
        %  	toFile( obj, FileName )
        %  
        %  	<Description>
        %  	store the linearization results in a struct, which is then
        %  	stored in a file
        %  
        %  	<Inputs>
        %  	>obj: the Result object
        %  	>FileName: the target file name
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial version 
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function res = ToStruct(obj)
        % TOSTRUCT store data in struct
        %  
        %  	<Syntax>
        %  	[ res ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	store the linearization results in a struct
        %  
        %  	<Inputs>
        %  	>obj: the Result object
        %  
        %  	<Outputs>
        %  	>res: struct containing the linearization results
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function h = plot(obj, varargin)
        % PLOT linearization results
        %  
        %   <Syntax>
        %   h = obj.plot(XValues, Function)
        %   h = obj.plot(XValues, Function, 'YValues', YValues)
        %   h = obj.plot(XValues, Function, 'PlotArguments', PlotArguments)
        %   h = obj.plot(XValues, Function, 'RemainingBreakpoints', RemainingBreakpoints)
        %  
        %  	<Description>
        %  	plot the linearization results over the grid breakpoints. The
        %  	first argument is the x-values over which the results are to be
        %  	plotted. The other values have either to be fixed via
        %  	'RemainingBreakpoints' (using the name of the breakpoint as
        %  	Name in the Name-Value pair). Then the closest index in the
        %  	breakpoint vector is chosen. Or, it can be set via 'YValues',
        %  	in which case a three dimensional plot is created, with the
        %  	the trim results on the z-axis.
        %  	the required Function argument has to be a function handle,
        %  	which takes the following arguments <x0, xdot0, u0, y0, sys,
        %  	trimmed> and computes a plottable quantity for every
        %  	linearization point (e.g. damping, frequency, etc.). if the
        %  	function outputs one argument, a 2D plot is created with the
        %  	function result over the XValues. if the function outputs two
        %  	arguments, a 3D plot is created with the XValue over the two
        %  	function outputs (e.g. real and imaginary parts of
        %  	eigenvalues). output arguments can have any length.
        %  
        %  	<Inputs>
        %  	>obj: the Result object
        %  	>XValues: name of breakpoint vector, which serves as X-Axis
        %  	>Function: function handle to compute plottable quantity
        %  	from linearization result
        %  
        %  	<NameValue>
        %  	>YValues: include this, to plot results over two different
        %  	breakpoints
        %  	>PlotArguments: cell array of arguments, that are directly
        %  	passed to the underlying plot function
        %  	>RemainingBreakpoints: names of remaining breakpoints to
        %  	fully define a point in the underlying ND-grid
        %  
        %  	<Outputs>
        %  	>h: plot handle
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function [ReturnValues, trimmed] = getValuesFromGridbyIndex(obj, Idx, varargin)
        % GETVALUESFROMGRIDBYINDEX get values from grid via indexing
        %  
        %  	<Syntax>
        %   [ReturnValues, trimmed] = obj.getValuesFromGridbyIndex( Idx )
        %   [ReturnValues, trimmed] = obj.getValuesFromGridbyIndex( Idx, DesiredProperty)
        %  
        %  	<Description>
        %  	get a subset of the result grid by accessing it via
        %  	indexing. The index can be a linear index, or a cell array
        %  	of indices, if the separate dimensions are to be accessed
        %  	individually. Additionally, the DesiredProperty input can
        %  	be used to access a specific property of the linearization
        %  	point (e.g. x0, u0, ...), by entering that property's name
        %  	as string
        %  
        %  	<Inputs>
        %  	>obj: the Result object
        %  	>Idx: index of the desired sub-grid (either as numeric
        %  	array, or cell array of numeric arrays)
        %  
        %  	<NameValue>
        %  	>DesiredProperty: name of desired property to extract from
        %  	linearization point object
        %  
        %  	<Outputs>
        %  	>ReturnValues: the extracted values from the grid, either
        %  	the complete linearization point objects are returned, or the
        %  	desired property in a cell array of appropriate size
        %  	>trimmed: logical array of appropriate size indicating if
        %  	the respective points had been trimmed
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function [ReturnValues, trimmed, Breakpoints, SelectIdx] = getValuesFromGridbyBreakpoints(obj, varargin)
        % GETVALUESFROMGRIDBYBREAKPOINTS get values of result grid by
        %  specifing desired breakpoints
        %  
        %  	<Syntax>
        %   [ReturnValues, trimmed, Breakpoints, SelectIdx] = ...
        %       obj.getValuesFromGridbyBreakpoints( 'BreakpointName1', BreakpointValues1, ..., DesiredProperty)
        %  
        %  	<Description>
        %  	extract the Result grid's values by defining desired
        %  	values for the breakpoints of this grid. The desired
        %  	breakpoints are entered as name-value pairs, where the names
        %  	have to match the breakpoint names of the grid. The values
        %  	can be any array of numeric values, where the closest match
        %  	in the underlying grid's breakpoints are returned (no
        %  	interpolation is performed).
        %  	All breakpoints, that are present in the grid have to be
        %  	specified, by alternating inputs of names and values.
        %  	Additionally, the last input can be used to access a specific
        %  	property of the linearization point (e.g. x0, u0, ...), by
        %  	entering that property's name as string
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>BreakpointNameXX: name of one breakpoint to be considered
        %  	>BreakpointValuesXX: desired values of the respective
        %  	breakpoint
        %  	>DesiredProperty: name of the linearization point property
        %  	to return as string
        %  
        %  	<Outputs>
        %  	>ReturnValues: the extracted values from the grid, either
        %  	the complete linearization point objects are returned, or the
        %  	desired property in a cell array of appropriate size
        %  	>trimmed: logical array of appropriate size indicating if
        %  	the respective points had been trimmed
        %  	>Breakpoints: struct array containing name and value
        %  	information about the breakpoints of the actually returned
        %  	data array
        %  	>SelectIdx: indices in grid, which were extracted
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = Result(Grid)
        % RESULT constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = Result( Grid )
        %  
        %  	<Description>
        %  	create a LinearizerResult object, containing the
        %  	linearization results in a falcon.trim.Grid. Size is derived
        %  	from the passed Grid
        %  
        %  	<Inputs>
        %  	>Grid: the grid, from which the result grid size is derived
        %  
        %  	<Outputs>
        %  	>obj: the newly created obj
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromStruct(s)
        % FROMSTRUCT load data from struct
        %  
        %  	<Syntax>
        %  	[ obj ] = falcon.trim.lin.Result.fromStruct( s )
        %  
        %  	<Description>
        %  	load linearization data from struct
        %  
        %  	<Inputs>
        %  	>s: struct containing data
        %  
        %  	<Outputs>
        %  	>obj: object containing the linearization results
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial version
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromFile(FileName)
        % FROMFILE load data from file
        %  
        %  	<Syntax>
        %  	[ obj ] = fromFile( FileName )
        %  
        %  	<Description>
        %  	load linearization data from file
        %  
        %  	<Inputs>
        %  	>FileName: Filename from which to load the data
        %  
        %  	<Outputs>
        %  	>obj: object containing the linearization results
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: final commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end