classdef (Abstract) TrimSolver < falcon.core.Handle & falcon.trim.core.HasTrimProblem
    % FALCON.TRIM.SOLVER.TRIMSOLVER solver class for trim problems
    %  
    %  	<Description>
    %  	this class implements all functions, which are common for all
    %  	solver algorithms. 
    %  
    %   Especially, it implements the strategy, which is used to trim a
    %   regular N-D grid in an efficient manner. To do so, a list of
    %   "border indices" is kept, which signify the border of the area,
    %   that has already been treated. Then the algorithm progresses as follows: 
    %       * The next point to trim is the point on the border, which has
    %       the smallest eucledian distance to the center of the grid (in
    %       'index' coordinates).
    %       * If it was trimmed succesfully, all its neighbors enter the
    %       border. 
    %       * If it cannot bet trimmed, the border does not change.
    %       * this is done, until all points on the border are found to be
    %       non-trimmable, or exceed the index range (if the complete N-D
    %       grid can be trimmed). 
    %  
    %   The algorithm starts in an 'init' phase, where the same idea is
    %   used, but the neighbors of the currently processed point enter the
    %   border no-matter-what. This init-phase is over, as soon as a point
    %   is found, that can be trimmed.
    %  
    %   Thus the trimmed region can be thought of as a balloon, that is
    %   inflated until it is completely contained within the trimmable
    %   envelope.
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
    end

    methods
        function delete(obj)
        % DELETE clearnup
        %  
        %  	<Syntax>
        %  	delete( obj )
        %  
        %  	<Description>
        %  	perform cleanup operations, e.g. make sure that waitbar is
        %  	closed
        %  
        %  	<Inputs>
        %  	>obj: the TrimSolver object
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial version
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function SolverOutput_array = solve(obj, varargin)
        % SOLVE solver call
        %  
        %   <Syntax>
        %   Result = obj.solve( )
        %   Result = obj.solve( TrimProblem )
        %  
        %  	<Description>
        %  	check that a problem to solve is available. Bake it, i.e.
        %  	perform consistency checks and indexing operations and call
        %  	child-class solver routine.
        %  	do so for every grid point, also start animation and waitbar
        %  	during grid trim
        %  
        %  	<Inputs>
        %  	>obj: the TrimSolver object
        %  
        %  	<NameValue>
        %  	>TrimProblem: TrimProblem object to be solved
        %  
        %  	<Outputs>
        %  	>SolverOutput_array: solver specific output struct
        %  
        %  	<ChangeLog>
        %  	2016-12-15 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-03-08 - Ch. Goettlicher: update to grid-trimming
        %  	2017-03-14 - Ch. Goettlicher: update waitbar every 1%
        %  	2017-11-09 - Ch. Goettlicher: implement trim strategy "from
        %                inside out"
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = TrimSolver()
        % TRIMSOLVER the constructor
        %  
        %  	<Syntax>
        %  	[ obj ] = TrimSolver( obj )
        %  
        %  	<Description>
        %  	check, if optimization toolbox is installed, and throw error
        %  	if fmincon or fsolve is used to solve problem
        %  
        %  	<Outputs>
        %  	>obj: newly created TrimSolver object
        %  
        %  	<ChangeLog>
        %  	2017-02-07 - Ch. Goettlicher: initial version
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function setTrimProblem(obj, TrimProblem)
        % SETTRIMPROBLEM set the trim problem
        %  
        %  	<Syntax>
        %  	setTrimProblem( obj, TrimProblem )
        %  
        %  	<Description>
        %  	set reference to a trim problem handle
        %  
        %  	<Inputs>
        %  	>obj: the HasTrimProblem object
        %  	>TrimProblem: the trim problem
        %  
        %  	<ChangeLog>
        %  	2016-12-15 - Ch. Goettlicher: initial version
        %  	2017-02-07 - Ch. Goettlicher: commenting
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

    end

end