classdef Parameter < falcon.trim.core.TVC & falcon.parest.Parameter
    % FALCON.TRIM.PARAMETER constructor for falcon.trim.Parameter object
    %  
    %  	<Description>
    %  	create an object, describing the model's parameter. Most parameter
    %  	will be passed to the underlying falcon.Control object
    %  
    %  	<ChangeLog>
    %  	2017-12-20 - Ch. Goettlicher: final commenting

    % -------------------------------------------------------------------------
    %                                FALCON.m
    % Copyright (c) 2014-2019 Institute of Flight System Dynamics, TUM, Munich
    % Matthias Bittner, Matthias Rieck, Maximilian Richter,
    % Benedikt Grueter, Johannes Diepolder, Florian Schwaiger,
    % Patrick Piprek, and Florian Holzapfel
    % Downloading, using, copying, or modifying FALCON.m code constitutes an
    % agreement to ALL of the terms of the FALCON.m EULA.
    % -------------------------------------------------------------------------

    properties
        % flag indicating if this value is known
        Known
        % the trim value
        TrimValue
        % trim grid
        TrimGrid
        % Flag to indicate if Bayesian estimation is to be used for this
        % parameter
        Bayesian
        % falcon.parest.Parameter/BayesianValue is a property.
        BayesianValue
        % base color for std-deviation and correlation plots
        BaseColor
        % parameter labels for figures
        Label
        % The current value of this parameter
        Value
        % Index the index of this parameter in the z-Vector
        Index
        % Scaling Parameters initialized with 1. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Scaling
        % Offset parameter which is initialized with 0. The offset
        % and scaling is assigned in the following way:
        %  
        % x_scaled = (x_unscaled - Offset) * Scaling
        Offset
        % Name of object.
        Name
        % Lower bound of the falcon.core.OVC value. It is initialized with
        % minus infinity. Scaling and Offset are applied to the
        % LowerBound as well.
        LowerBound
        % Upper bound of the falcon.core.OVC value. It is initialized with
        % plus infinity. Scaling and Offset are applied to the
        % UpperBound as well.
        UpperBound
        % Whether this value is fixed or not.
        isFixed
        % standard deviation of this parameter
        StandardDeviation
        % relativ standard deviation, i.e. sigma/value
        RelStandardDeviation
    end

    methods
        function loadValuesFromFile(obj, FullFileName)
        % LOADVALUESFROMFILE load values from file
        %  
        %  	<Syntax>
        %  	loadValuesFromFile( obj, FullFileName )
        %  
        %  	<Description>
        %  	use the superclass method to load the parameter values from a
        %  	file, where only the values that can be found in the file are
        %  	set. The values are transfered to the TrimValue property and
        %  	the actual value property is filled with nan's
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>FullFileName: filename to load data from
        %  
        %  	<ChangeLog>
        %  	2017-03-14 - Ch. Goettlicher: initial design
        %  	2017-03-14 - Ch. Goettlicher: take care of values not in file
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = Parameter(ParameterName, varargin)
        % PARAMETER constructor for falcon.trim.Parameter object
        %  
        %  	<Syntax>
        %   obj = Parameter(ParameterName)
        %   obj = Parameter(ParameterName, LowerBound)
        %   obj = Parameter(ParameterName, LowerBound, UpperBound)
        %   obj = Parameter(ParameterName, LowerBound, UpperBound, Scaling)
        %   obj = Parameter(ParameterName, LowerBound, UpperBound, Scaling, Offset)
        %   obj = Parameter(ParameterName, LowerBound, UpperBound, Scaling, Offset, TrimValue)
        %   obj = Parameter(..., 'Name', Value)
        %  
        %  	<Description>
        %  	create an object, describing the model's parameter. Most
        %  	parameter will be passed to the underlying falcon.Control
        %  	object
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>varargin: 
        %  	>ParameterName: name of for this value
        %  
        %  	<NameValue>
        %  	>LowerBound: lower bound for this values
        %  	>UpperBound: upper bound for this value
        %  	>Scaling: scaling to be used during trimming
        %  	>Offset: offset to be used during trimming (unused so far)
        %  	>TrimValue: trim value; can be array to trim several
        %  	parameter values
        %  
        %  	<Outputs>
        %  	>obj: the parameter object
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial version
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromStruct(s)
        % FROMSTRUCT load data from struct
        %  
        %   <Syntax>
        %   obj = falcon.trim.core.Parameter.fromStruct( s )
        %  
        %  	<Description>
        %  	load TVC data from struct
        %  
        %  	<Inputs>
        %  	>s: struct containing data
        %  
        %  	<Outputs>
        %  	>obj: object containing the parameter data
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial version 
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromModel(ModelHandle)
        % FROMMODEL create parameter array from model information
        %  
        %   <Syntax>
        %   obj = falcon.trim.Parameter.fromModel( ModelHandle )
        %  
        %  	<Description>
        %  	create an array of falcon.trim.Parameter objects from the
        %  	names supplied by the simulation model builder model
        %  	'ModelHandle'. The parameters are all created by supplying
        %  	only the name to the constructor, i.e. all other properties
        %  	are set to their default values (Known = true, lb = -inf,
        %  	ub = +inf, ...).
        %  	
        %  	for the actual creation of parameters, the
        %  	falcon.trim.core.TVC.fromModel_lowLevel method is used.
        %  
        %  	<Inputs>
        %  	>ModelHandle: valid handle to simulation model builder mex
        %  	function
        %  
        %  	<Outputs>
        %  	>obj: array of falcon.trim.parameter objects
        %  
        %  	<ChangeLog>
        %  	2017-11-01 - Ch. Goettlicher: initial version 
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function toFile(obj, FileName)
        % TOFILE store data in file
        %  
        %  	<Syntax>
        %  	toFile( obj, FileName )
        %  
        %  	<Description>
        %  	store the TVC data in a struct, which is then stored in a
        %  	file
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>FileName: the name of the file, from which the data is to
        %  	be loaded
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function s = toTrimStruct(obj)
        % TOTRIMSTRUCT Create a struct from this object.
        %  
        %  	<Syntax>
        %  	[ s ] = toTrimStruct( obj )
        %  
        %  	<Description>
        %  	This metod creates a struct of this object including the
        %  	fields of the parent OVC and this TVC properties
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  
        %  	<Outputs>
        %  	>s: struct containing the inherent properties of this object.
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function value = getGridValue(obj, idx)
        % GETGRIDVALUE get values corresponding to serial idx
        %  
        %  	<Syntax>
        %  	[ value ] = getGridValue( obj, idx )
        %  
        %  	<Description>
        %  	get the values from the grid in serialized fashion; output
        %  	array, if an array of objects is used for the call
        %  
        %  	<Inputs>
        %  	>obj: the TVC object
        %  	>idx: linear indices to extract from the grid
        %  
        %  	<Outputs>
        %  	>value: extracted values
        %  
        %  	<ChangeLog>
        %  	2017-03-10 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function set(obj, name, prop, val)
        % SET certain property by name
        %  
        %  	<Syntax>
        %  	set( obj, name, prop, val )
        %  
        %  	<Description>
        %  	use this method to set a property in an array of TVCs by name
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>name: name of the TVC to manipulate
        %  	>prop: the name of the property to be set
        %  	>val: the value to be set
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function GridPlot(obj, varargin)
        % plot grid values for this TVC value
        %  
        %   <Syntax>
        %   obj.GridPlot(XValue, 'YValues', YValues)
        %   obj.GridPlot(XValue, 'PlotArguments', PlotArguments)
        %   obj.GridPlot(XValue, 'RemainingBreakpoints', RemainingBreakpoints)
        %  
        %  	<Description>
        %  	plot the values of this TVC over the grid breakpoints. The
        %  	first argument is the x-values over which this TVC is to be
        %  	plotted. The other values have either to be fixed via
        %  	'RemainingBreakpoints' (using the name of the breakpoint as
        %  	Name in the Name-Value pair). Then the closest index in the
        %  	breakpoint vector is chosen. Or, it can be set via 'YValues',
        %  	in which case a three dimensional plot is created, with the
        %  	values of this TVC on the z-axis.
        %  
        %  	<Inputs>
        %  	>obj: 
        %  	>varargin: 
        %  	>XValue: value over which to plot (e.g. altitude, speed,
        %  	etc.)
        %  
        %  	<NameValue>
        %  	>YValues: if this is value is included, a 3D plot is
        %  	generated using XValue, YValues and the grid values on the
        %  	z-axis
        %  	>PlotArguments: cell array of arguments, that are directly
        %  	passed to the underlying plot function
        %  	>RemainingBreakpoints: names of remaining breakpoints to
        %  	fully define a point in the underlying ND-grid
        %  
        %  	<ChangeLog>
        %  	2017-03-08 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function obj = fromFile(FileName)
        % FROMFILE load TVC data from file
        %  
        %   <Syntax>
        %   obj = falcon.trim.core.TVC.fromFile( FileName)
        %  
        %  	<Description>
        %  	load TVC data from file
        %  
        %  	<Inputs>
        %  	>FileName: Filename from which to load the data
        %  
        %  	<Outputs>
        %  	>obj: the newly created TVC object
        %  
        %  	<ChangeLog>
        %  	2017-03-13 - Ch. Goettlicher: initial design
        %  	2017-12-20 - Ch. Goettlicher: final commenting
        end

        function Covariance = getBayesianCovariance(obj)
        % falcon.parest.Parameter/getBayesianCovariance is a function.
        %   Covariance = getBayesianCovariance(obj)
        end

        function storeValueAsBayesian(obj)
        % falcon.parest.Parameter/storeValueAsBayesian is a function.
        %   storeValueAsBayesian(obj)
        end

        function setBayesian(obj, isBayesian)
        % falcon.parest.Parameter/setBayesian is a function.
        %   setBayesian(obj, isBayesian)
        end

        function setFixed(obj, isFixed)
        % SETFIXED set parameters fixed
        %  
        %  	<Syntax>
        %  	setFixed( obj, isFixed )
        %  
        %  	<Description>
        %  	set falcon.parest.Parameter.isFixed value. if a scalar is
        %  	input to this function, all elements of the parameter array
        %  	are assigned with this value. If a logical array of proper
        %  	size is input, the parameters are set fixed accordingly.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>isFixed: logical scalar or array of proper size. new value
        %  	of falcon.parest.Parameter.isFixed property
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-22 - Ch. Goettlicher: bug fix for ND obj arrays
        end

        function setValue(obj, Value)
        % SETVALUE set new value without storing
        %  
        %  	<Syntax>
        %  	setValue( obj, Value )
        %  
        %  	<Description>
        %  	set new value for this parameter array
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: new parameter value
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setValueByName(obj, Value, Name)
        % SETVALUEBYNAME set a parameter value by name
        %  
        %  	<Syntax>
        %  	setValueByName(obj, Value, Name)
        %  
        %  	<Description>
        %  	set the provided Values for the parameters provided in the
        %  	cell array Name
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: the new parameter values
        %   >Name: cell array of parameter names
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function updateValue(obj, Value)
        % UPDATEVALUE set new value and store
        %  
        %  	<Syntax>
        %  	updateValue( obj, Value )
        %  
        %  	<Description>
        %  	change the value of the parameter and store it to the
        %  	falcon.parest.Parameter.Evolution property
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>Value: the new parameter value
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function [s, cov] = ToStruct(obj)
        % TOSTRUCT store parameter information in struct
        %  
        %  	<Syntax>
        %  	[ s, cov ] = ToStruct( obj )
        %  
        %  	<Description>
        %  	store all information of this parameter object in a struct.
        %  	Can be used to store the parameter data permanently, even if
        %  	the class definition changes (which would make storing the
        %  	complete object of a former version useless)
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>s: the parameter's information in a struct.
        %  	>cov: the parameter's covariance, for the non-fixed
        %  	parameter
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-02-07 - Ch. Goettlicher: bugfix for fixed parameters
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function ToFile(obj, FullFileName)
        % TOFILE store parameter data to file
        %  
        %  	<Syntax>
        %  	ToFile( obj, FullFileName )
        %  
        %  	<Description>
        %  	convert parameter data to struct and store this struct to a
        %  	file. the conversion to a struct makes the stored file
        %  	independent of the version of the class in use.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>FullFileName: full path and filename of the file to store
        %  	the parameter data in
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function obj = FromFile(FullFileName)
        % FROMFILE load parameter from file
        %  
        %  	<Syntax>
        %  	[ obj ] = falcon.parest.Parameter.FromFile( FullFileName )
        %  
        %  	<Description>
        %  	create a new falcon.parest.Parameter object from a struct
        %  	stored in a file
        %  
        %  	<Inputs>
        %  	>FullFileName: full path and file name to the file
        %  	containing the Parameter data
        %  
        %  	<Outputs>
        %  	>obj: the newly created Parameter object
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setLabels(obj, Labels)
        % SETLABELS set parameter labels
        %  
        %   <Syntax>
        %   obj.setLabels( Labels )
        %  
        %   <Description>
        %   set falcon.parest.Parameter.Label value by passing a cell
        %   array of strings of proper size to this function. If this
        %   method is called for an array of parameters, the labels
        %   will be assigned to parameters accordingly.
        %   <Inputs>
        %   >Labels: cell array of strings containing the label of the
        %   parameters
        %  
        %  	<ChangeLog>
        %   2017-12-19 - X. Fank: initial design
        %  	2017-01-28 - Ch. Goettlicher: moved label things
        end

        function plotCorrelationMatrix(obj)
        % PLOTCORRELATIONMATRIX 3D bar plot of correlation matrix
        %  
        %  	<Syntax>
        %  	plotCorrelationMatrix( obj )
        %  
        %  	<Description>
        %  	creates a 3D bar plot of the absolute value of the
        %  	correlation matrix. The larger the values, the more red the
        %  	bars become.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: sync with Goettlicher TIKZ
        %   exporter
        end

        function plotBar(obj, varargin)
        % PLOTBAR make bar plot with parameter values
        %  
        %  	<Syntax>
        %  	plotBar( obj )
        %  	plotBar( obj, Name, Value )
        %  
        %  	<Description>
        %  	create a bar plot with the parameter values, standard
        %  	deviations, and possibly initial and true values
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >plotInitialValues: integer to indicate if initial values
        %   are to be plotted, too (1), or if the initial values after
        %   refinement are to be plotted (2).
        %   >trueValues: array of true parameter values for simulation
        %   examples
        %   >nSigma: integer to indicate how many standard deviations
        %   shall be used for the errorbar plot
        %  
        %  	<ChangeLog>
        %  	2018-06-26 - Ch. Goettlicher: initial design
        end

        function plotRelativeStandardDeviation(obj, varargin)
        % PLOTRELATIVESTANDARDDEVIATION bar plot of relative standard
        %   deviation
        %  
        %  	<Syntax>
        %  	plotRelativeStandardDeviation( obj )
        %  	plotRelativeStandardDeviation( obj, InitialValueFlag )
        %  
        %  	<Description>
        %  	creates a horizontal bar plot of the relative standard
        %  	deviations in the current axis
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %   <NameValue>
        %   >InitialValueFlag: flag to indicate, if initial value is to
        %   be plotted alongside
        %  
        %  	<ChangeLog>
        %   2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %  	2018-01-19 - Ch. Goettlicher: added initial value
        end

        function plotParameterEvolution(obj, varargin)
        % PLOTPARAMETEREVOLUTION short description
        %  
        %  	<Syntax>
        %  	plotParameterEvolution( obj )
        %  	plotParameterEvolution( obj, 'Name', Value )
        %  
        %  	<Description>
        %  	plot the evolution of the parameter values during the
        %  	iteration. the last value and its standard deviation are
        %  	highlighted with a red line and an error bar.
        %   if any improvement algorithm on the initial guess was used,
        %   these improvements will be highlightes with circles
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<NameValue>
        %  	>SubplotConfig: two element vector containing the number of
        %  	vertical and horizontal subplots to create
        %  	>Normalized: logical indicating if the true parameter value
        %  	or its normalized history is to be plotted
        %   >showDifference: show difference to initial value in
        %   percent to illustrate the change during the optimization
        %  
        %  	<ChangeLog>
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        %   2018-06-26 - Ch. Goettlicher: added initial improvement
        %   marker; adjusted parameter labels
        end

        function self = setValueAndRange(self, value, range)
        % falcon.Parameter/setValueAndRange is a function.
        %   [self] = setValueAndRange(self, value, range)
        end

        function self = setValueAndBounds(self, value, lowerBound, upperBound)
        % falcon.Parameter/setValueAndBounds is a function.
        %   [self] = setValueAndBounds(self, value, lowerBound, upperBound)
        end

        function obj = setOffset(obj, offset)
        % Set the offset of this object.
        %  
        % <Syntax>
        % obj.setOffset(offset)
        %  
        % <Description>
        % Set the offset of this object. Please note
        % that the input must be a real, scalar value.
        %  
        % <Inputs>
        % >offset: Offset of this object.
        %  
        % <Keywords>
        % OVC!Offset
        end

        function obj = setScaling(obj, scaling)
        % Set the scaling of this object.
        %  
        % <Syntax>
        % obj.setScaling(scaling)
        %  
        % <Description>
        % The input must be a real positive scalar value and should
        % scale the value of this object to a range between -1 and 1.
        %  
        % <Inputs>
        % >scaling: Scaling of the this object.
        %  
        % <Keywords>
        % OVC!Scaling
        end

        function element = findFirst(self, pattern)
        % Find the first element matching a regex.
        %  
        % <Syntax>
        % element = array.findFirst(pattern)
        %  
        % <Description>
        % Extract the first element from the array whose Name attribute
        % matches the given pattern, as determined by the regexp function.
        % If no element matches, an exception is thrown.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > element: the first matching element
        end

        function elements = find(self, pattern)
        % Find elements matching a regex.
        %  
        % <Syntax>
        % elements = array.find(pattern)
        %  
        % <Description>
        % Extract object array elements whose Name attributes match the
        % given pattern, as determined by the regexp function.  The
        % returned array is empty if no element matches.
        %  
        % <Inputs>
        % > pattern: see documentation for regexp
        %  
        % <Outputs>
        % > elements: the entries of the object array whose names match
        end

        function bool = ne(varargin)
        % ~= (NE)   Not equal relation for handles.
        %   Handles are equal if they are handles for the same object and are 
        %   unequal otherwise.
        %  
        %   H1 ~= H2 performs element-wise comparisons between handle arrays H1 
        %   and H2.  H1 and H2 must be of the same dimensions unless one is a 
        %   scalar.  The result is a logical array of the same dimensions, where 
        %   each element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = NE(H1, H2) stores the result in a logical array of the same
        %   dimensions.
        end

        function bool = eq(varargin)
        % == (EQ)   Test handle equality.
        %   Handles are equal if they are handles for the same object.
        %  
        %   H1 == H2 performs element-wise comparisons between handle arrays H1 and
        %   H2.  H1 and H2 must be of the same dimensions unless one is a scalar.
        %   The result is a logical array of the same dimensions, where each
        %   element is an element-wise equality result.
        %  
        %   If one of H1 or H2 is scalar, scalar expansion is performed and the 
        %   result will match the dimensions of the array that is not scalar.
        %  
        %   TF = EQ(H1, H2) stores the result in a logical array of the same 
        %   dimensions.
        end

        function map = byNameUnique(self)
        % Get a struct for name based access to the object array, allowing
        % only unique names.
        %  
        % <Syntax>
        % [ map ] = obj.byNameUnique()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  Throws
        % an error in case there are any duplicate names.
        %  
        % <Keywords>
        % OVC!byName!Unique
        end

        function map = byName(self)
        % Get a struct for name based access to the object array.
        %  
        % <Syntax>
        % [ map ] = obj.byName()
        %  
        % <Description>
        % The method returns all entries from the object array 'obj' as
        % fields of a struct, with the object Name as field name.  In case
        % there are duplicate names, the output struct contains object
        % arrays.
        %  
        % <Keywords>
        % OVC!byName
        end

        function self = relaxRange(self, range)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = relaxBounds(self, lowerBound, upperBound)
        % Relax the lower and upper bound.
        %  
        % <Syntax>
        % obj.relaxBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [min(a, c), max(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = tightenRange(self, range)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenRange(range)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > range: two-element double vector
        end

        function self = tightenBounds(self, lowerBound, upperBound)
        % Tighten the lower and upper bound.
        %  
        % <Syntax>
        % obj.tightenBounds(lowerBound, upperBound)
        %  
        % <Description>
        % For current bounds [a, b] and given bounds [c, d], set the bounds
        % to [max(a, c), min(b, d)].
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function self = setRange(self, range)
        % Set the lower and upper bound using a vector argument.
        %  
        % <Syntax>
        % obj.setRange(range)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > range: a two-element double vector
        end

        function self = setBounds(self, lowerBound, upperBound)
        % Set the lower and upper bound.
        %  
        % <Syntax>
        % obj.setBounds(lowerBound, upperBound)
        %  
        % <Description>
        % Set the lower and upper bound.
        %  
        % <Inputs>
        % > lowerBound: scalar double
        % > upperBound: scalar double
        end

        function obj = setUpperBound(obj, upperBound)
        % Set the upper bound of this object.
        %  
        % <Syntax>
        % obj.setUpperBound(upperBound)
        %  
        % <Description>
        % Set the upper bound of this object. The input must be a real, scalar
        % value and bigger than the lower bound.
        %  
        % <Inputs>
        % >upperBound: Upper bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Upper
        end

        function obj = setLowerBound(obj, lowerBound)
        % Set the lower bound of this object.
        %  
        % <Syntax>
        % obj.setLowerBound(lowerBound)
        %  
        % <Description>
        % Set the lower bound of this object. The input must be a real, scalar
        % scalar and smaller than the upper bound.
        %  
        % <Inputs>
        % >lowerBound: Lower bound of this object.
        %  
        % <Keywords>
        % OVC!Bound!Lower
        end

        function Correlation = getCorrelation(obj)
        % GETCORRELATION get correlation matrix for an array of
        %   parameters
        %  
        %  	<Syntax>
        %  	[ Correlation ] = getCorrelation( obj )
        %  
        %  	<Description>
        %  	get the correlation matrix for an array of parameters. It is
        %  	computed from falcon.parest.Parameter.getCovariance and the
        %  	corresponding standard deviations
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>Correlation: correlation matrix
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function setVariances(obj, CrossVariance, varargin)
        % SETVARIANCES set parameter variances
        %  
        %  	<Syntax>
        %  	setVariances( obj, CrossVariance, CrossVarianceParameters )
        %  
        %  	<Description>
        %  	store the cross variances with other parameters. Both the
        %  	cross variances and the respective parameters are stored.
        %  	However these two arrays are sorted such, that the first
        %  	entry always is the parameter itself, i.e. the first
        %  	covariance is the parameter variance.
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  	>CrossVariance: double array of cross variance parameters
        %  	>CrossVarianceParameters: Parameter objects which are
        %  	associated with the corresponding entries in the
        %  	CrossVariance input
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

        function Covariance = getCovariance(obj)
        % GETCOVARIANCE get covariance matrix of a parameter array
        %  
        %  	<Syntax>
        %  	[ Covariance ] = getCovariance( obj )
        %  
        %  	<Description>
        %  	obtain the covariance matrix for an array of parameters. to
        %  	do so, the Cross-Variance-Parameters are compared to the
        %  	parameter array (obj), and the covariance matrix is built up
        %  	from the cross variances
        %  
        %  	<Inputs>
        %  	>obj: the Parameter object
        %  
        %  	<Outputs>
        %  	>Covariance: Covariance matrix for this array of parameters
        %  
        %  	<ChangeLog>
        %  	2017-01-28 - Ch. Goettlicher: initial design
        %  	2017-12-19 - Ch. Goettlicher: commenting (-->EMT completion)
        end

    end

end