clear 
close all

%% Create States
x = [falcon.State('x1',     -inf,   inf,    1.0);...
     falcon.State('x2',     -inf,   inf,    1.0)];
                                                     
%% Create Control
u = falcon.Control('u', -1, 1, 1.0, 'Fixed',true);

%% Create Outputs
y = [falcon.Output('y1', -inf, inf, 1.0);...
     falcon.Output('y2', -inf, inf, 1.0)];

%% Create Parameters
p = [falcon.Parameter('D'         ,1            ,0.001      ,10         ,1.0);...
     falcon.Parameter('omega_0'   ,1.5          ,0.001      ,10         ,1.0);...
     falcon.Parameter('k_s'       ,1            ,0.001      ,10         ,1.0)];

%% Build model
if ~exist(['pt2.', mexext()], 'file')
    % create the simulation model object
    mdl_build = falcon.SimulationModelBuilder('pt2', x, u, p, @Model_PT2);
    % set the outputs
    mdl_build.setOutputs( y );
    % build the model
    mdl_build.Build();
end


%% Load meassurement data
load Pt2Data.mat

% Create Phases in a for loop
nPhases = numel(t_data);

for iPhase = 1:nPhases
    t_data{iPhase} = t_data{iPhase}(:, 1:end);
    u_data{iPhase} = u_data{iPhase}(:, 1:end);
    z_data{iPhase} = z_data{iPhase}(:, 1:end);
end

%% Build least squares cost

if ~exist(strcat('LsMayerCost.', mexext()), 'file')
    % Rebuild the cost function model.  The calculations are performed using
    % Derivative Subsystems to speed up the build process compared to symbolic
    % derivatives.

    lc_build = falcon.PointConstraintBuilder('LsMayerCost');

    costParts = {};
    for k = 1 : nPhases

        N = size(t_data{k}, 2);

        zName = sprintf('z_%d', k);

        lc_build.addPhaseInput(y, u, N);
        lc_build.addConstantInput(zName, [numel(y), N]);

        yName = sprintf('outputs_g%d', k);
        cName = strcat('c_', yName);
        lc_build.addDerivativeSubsystem( ...
            @ls_Mayer_square_res, ...
            {yName, zName}, ...
            {cName}, ...
            'OutputSizes', {[1, 1]}, ...
            'OutputJacobianSparsity', {ones([1, numel(y) * N])});
        costParts{end + 1} = cName;  %#ok<SAGROW>

        uName = sprintf('controls_g%d', k);
        cName = strcat('c_', uName);
        lc_build.addDerivativeSubsystem( ...
            @ls_Mayer_square, ...
            {uName}, ...
            {cName}, ...
            'OutputSizes', {[1, 1]}, ...
            'OutputJacobianSparsity', {ones([1, numel(u) * N])});
        costParts{end + 1} = cName;  %#ok<SAGROW>

    end

    lc_build.setParameters(p);
    lc_build.addDerivativeSubsystem( ...
        @ls_Mayer_square, ...
        {'parameters'}, ...
        {'c_p'}, ...
        'OutputSizes', {[1, 1]}, ...
        'OutputJacobianSparsity', {ones([1, numel(p)])});

    lc_build.addSubsystem( ...
        @(varargin) sum([varargin{:}]), ...
        'Inputs', vertcat(costParts(:), 'c_p'), ...
        'Outputs', {'c'});

    lc_build.setConstraintValueNames('c');

    lc_build.Build();

end


%% Setup parameter estimation problem
Scaling         = 1;
OutputWeighting = eye(2);

% Create Problem
Problem = falcon.Problem('pt2');


for iPhase = 1:nPhases
    % StateGrid
    tau{iPhase} = linspace(0, 1, size(t_data{iPhase},2));
    phase(iPhase) = Problem.addNewPhase(@pt2,  x, tau{iPhase}, t_data{iPhase}(1), t_data{iPhase}(end));

    % fix initial conditions at zero
    phase(iPhase).setInitialBoundaries(x, [0;0])
                        
    % ControlGrid
    controlgrid = phase(iPhase).addNewControlGrid(u);
    controlgrid.setValues(tau{iPhase}, u_data{iPhase});
    
    % Set the parameters required in the model
    phase(iPhase).Model.setModelParameters(p);
    
    % set the outputs required in the model
    phase(iPhase).Model.setModelOutputs( y );
end

% add the Cost as Mayer Cost
myMayerCost = Problem.addNewMayerCost(@LsMayerCost, falcon.Cost('Cost'), phase(1), tau{1}, phase(2), tau{2});
myMayerCost.setParameters(p); 
myMayerCost.addConstants( z_data{:} )

    
%% Prepare and solve the problem
Problem.Bake();
solver = falcon.solver.ipopt(); 
solver.setProblem(Problem);
solver.Solve();

%% Visualize
CreatePlots();