% script to generate example experimental data to test system
% identification solver - Boeing B777 lateral motion example

close all
clear
clc

%% ************************************************************************
% Step:    
% add necessary paths - use relative paths!
% *************************************************************************

falcon.init();

BasePath = fileparts(mfilename('fullpath'));
addpath(fullfile(BasePath, 'Model'));

% generate data directory if non-existent
if ~exist( fullfile(BasePath, 'Data'), 'dir' )
    mkdir(fullfile(BasePath, 'Data'));
end
addpath(fullfile(BasePath, 'Data'));

% setup model - i.e. build anew, if it has changed
buildB777LatModel

%% ************************************************************************
% Step:    
% setup model parameters
% *************************************************************************

N_r = -0.8109;
N_beta = 5.2575;
N_p = 0.2145;
N_xi = 0.0276;
N_zeta = -1.7843;
Y_r = 0.0173;
Y_beta = -0.2462;
Y_p = -0.0081;
Y_xi = -0.0043;
Y_zeta = 0.0318;
L_r = 1.2543;
L_beta = -12.2119;
L_p = -4.3412;
L_xi = -3.3192;
L_zeta = 1.2158;

p = [
    N_r;
    N_beta;
    N_p;
    N_xi;
    N_zeta;
    Y_r;
    Y_beta;
    Y_p;
    Y_xi;
    Y_zeta;
    L_r;
    L_beta;
    L_p;
    L_xi;
    L_zeta;
    ];


%% ************************************************************************
% Step:    
% setup model
% *************************************************************************

% define measurement noise covariance matrix
NoiseCov = diag([0.07; 0.03; 0.07; 0.03].^2);
NanPeriods = [2; 0; 2; 0]; % 1 means every sample, 2 means every 2nd sample etc.
% NanPeriods = [0; 0; 0; 0]; % 0 means no sample, 1 means every sample, 2 means every 2nd sample etc. is a NAN
rng(42); % set random number generator to a defined state

% time step
delta_t = 0.01;

modelName  = 'B777Lat';
modelFunc = str2func(modelName);
modelInfo  = modelFunc();

%% ************************************************************************
% Step:    
% trim model
% *************************************************************************

x0 = zeros(4, 1);

%% ************************************************************************
% Step:    
% perform experiments; for every experiment: 
%   1. increment index
%   2. create time vector
%   3. create inputs
%   4. use simulation model to generate clean data
%   5. add normally distributed, white noise
%   6. store data in experiment object
%   7. configure experiment tags
% *************************************************************************

iExp = 0;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 15;
t = 0:delta_t:t_end;

A = 0.3;
u = zeros(2, length(t));
u( 2, t>3 & t<=4.5) = A;
u( 2, t>4.5 & t<=6) = -A;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'zeta';
experiments(iExp).Tags.Type = 'doublet';
experiments(iExp).Tags.Iteration = 1;
experiments(iExp).Tags.ExperimentNumber = iExp;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 15;
t = 0:delta_t:t_end;

A = -0.35;
u = zeros(2, length(t));
u( 2, t>3 & t<=4) = A;
u( 2, t>4 & t<=5) = -A;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'zeta';
experiments(iExp).Tags.Type = 'doublet';
experiments(iExp).Tags.Iteration = 2;
experiments(iExp).Tags.ExperimentNumber = iExp;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 15;
t = 0:delta_t:t_end;

A = 0.7;
u = zeros(2, length(t));
u( 1, t>3 & t<=4) = A;
u( 1, t>4 & t<=6) = -A;
u( 1, t>6 & t<=7) = A;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'xi';
experiments(iExp).Tags.Type = 'bank to bank';
experiments(iExp).Tags.Iteration = 1;
experiments(iExp).Tags.ExperimentNumber = iExp;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 15;
t = 0:delta_t:t_end;

A = -0.5;
u = zeros(2, length(t));
u( 1, t>3 & t<=4.5) = A;
u( 1, t>4.5 & t<=7.5) = -A;
u( 1, t>7.5 & t<=9) = A;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'xi';
experiments(iExp).Tags.Type = 'bank to bank';
experiments(iExp).Tags.Iteration = 2;
experiments(iExp).Tags.ExperimentNumber = iExp;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 35;
t = 0:delta_t:t_end;

A_zeta = 0.3;
A_xi1 = -0.31;
A_xi2 = 0.3;
u = zeros(2, length(t));
u( 2, t>3 & t<=27) = A_zeta;
u( 1, t>4 & t<=26) = A_xi1;
u( 1, t>28 & t<=29.5) = A_xi2;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'xi,zeta';
experiments(iExp).Tags.Type = 'steady heading sideslip';
experiments(iExp).Tags.Iteration = 1;
experiments(iExp).Tags.ExperimentNumber = iExp;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 90;
t = 0:delta_t:t_end;

A = -0.3;
u = zeros(2, length(t));
u( 1, t>3 & t<=3.5) = A;

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));
for ii = 1:size(z,1)
    z(ii, 1:NanPeriods(ii):end) = nan();
end

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).x0 = x0;
experiments(iExp).Tags.Direction = 'Lat';
experiments(iExp).Tags.Control = 'xi';
experiments(iExp).Tags.Type = 'pulse';
experiments(iExp).Tags.Iteration = 1;
experiments(iExp).Tags.ExperimentNumber = iExp;

%% ************************************************************************
% Step:    
% set measurement names
% *************************************************************************

experiments.setMeasurementNames( modelInfo.output(2).argnames );
experiments.setMeasurementLabels( {'r [rad/s]', '\beta [rad]', 'p [rad/s]', '\phi [rad]'} );
experiments.setControlNames( modelInfo.input(2).argnames );
experiments.setControlLabels( {'\xi [-]', '\zeta [-]'} );

%% ************************************************************************
% Step:    
% plot experiment data
% *************************************************************************

figure('units', 'normalized', 'outerposition', [0 0 1 1])
experiments.plotExperiment('Tags', {'Control', 'Type', 'Iteration'})

%% ************************************************************************
% Step:    
% store data to disc
% *************************************************************************

experiments.ToFile(fullfile(BasePath, 'Data', 'data_B777Lat.mat'))
