% introduction example to using falcon.m in system identification.
% estimate parameters of a linear model of a Boeing B777's lateral
% movement.
%
% the artificial measurement data for this example is created using the
% script generateDataB777.m

falcon.init();

if ~exist('./Data/data_B777Lat.mat', 'file')
    generateDataB777();
end

% close all
% clear
clc

%% ************************************************************************
% Step:    
% add necessary paths - use relative paths!
% *************************************************************************

BasePath = fileparts(mfilename('fullpath'));
addpath( fullfile(BasePath, 'Model') )
addpath( fullfile(BasePath, 'Data') )

% setup model - i.e. build anew, if it has changed
% buildAcLatModel

%% ************************************************************************
% Step:    
% define ID states, controls, outputs, and parameters
% *************************************************************************

states   = [...
    falcon.parest.State('r');
    falcon.parest.State('beta');
    falcon.parest.State('p');
    falcon.parest.State('phi');
    ];

controls = [...
    falcon.parest.Control('xi');
    falcon.parest.Control('zeta');
    ];

IdParameters = [...
    falcon.parest.Parameter('N_r',    'Value',   -0.8109, 'Fixed', false);...
    falcon.parest.Parameter('N_beta', 'Value',    5.2575, 'Fixed', false);...
    falcon.parest.Parameter('N_p',    'Value',    0.2145, 'Fixed', false);...
    falcon.parest.Parameter('N_xi',   'Value',    0.0276, 'Fixed', false);...
    falcon.parest.Parameter('N_zeta', 'Value',   -1.7843, 'Fixed', false);...
    falcon.parest.Parameter('Y_r',    'Value',    0.0173, 'Fixed', false);...
    falcon.parest.Parameter('Y_beta', 'Value',   -0.2462, 'Fixed', false);...
    falcon.parest.Parameter('Y_p',    'Value',   -0.0081, 'Fixed', false);...
    falcon.parest.Parameter('Y_xi',   'Value',   -0.0043, 'Fixed', false);...
    falcon.parest.Parameter('Y_zeta', 'Value',    0.0318, 'Fixed', false);...
    falcon.parest.Parameter('L_r',    'Value',    1.2543, 'Fixed', false);...
    falcon.parest.Parameter('L_beta', 'Value',  -12.2119, 'Fixed', false);...
    falcon.parest.Parameter('L_p',    'Value',   -4.3412, 'Fixed', false);...
    falcon.parest.Parameter('L_xi',   'Value',   -3.3192, 'Fixed', false);...
    falcon.parest.Parameter('L_zeta', 'Value',    1.2158, 'Fixed', false);...
    ];
IdParameters.setLabels({'N_r', 'N_\beta', 'N_p', 'N_\xi', 'N_\zeta', ...
    'Y_r', 'Y_\beta', 'Y_p', 'Y_\xi', 'Y_\zeta', ...
    'L_r', 'L_\beta', 'L_p', 'L_\xi', 'L_\zeta'})
IdParameters_trueValues = [IdParameters.Value];

outputs = [...
    falcon.parest.Output('y_r'); ...
    falcon.parest.Output('y_beta', 'isConsidered', true, 'Scaling', 1, 'Offset', 0); ...
    falcon.parest.Output('y_p', 'isConsidered', false); ...
    falcon.parest.Output('y_phi'); ...
    ];

% use randomly distorted version of true values for non-fixed parameters 
rng(42); % set random number generator to a defined state
isFixed = [IdParameters.isFixed];
IdParameters(~isFixed).setValue( 3*rand(size(IdParameters(~isFixed))).* [IdParameters(~isFixed).Value].' );

IdParameters(~isFixed).setValue( 0*sign([IdParameters(~isFixed).Value]) );

%% ************************************************************************
% Step:    
% load and configure experiments
% *************************************************************************

experiments = falcon.parest.Experiment.FromFile('data_B777Lat');

% set initial conditions fixed / to be estimated
for iExp = experiments.'
%     % set selected ones fixed
%     iExp.x0.setFixed([...
%         false, ...  % r
%         false, ...  % beta
%         false, ...  % p
%         false, ...  % phi
%         ]);
    
    % set all fixed
    iExp.x0.setFixed(false);
end

% configure experiments
%   * use only first 4 experiments
Filter = falcon.parest.ExperimentFilter();           % setup filter
Filter.setFilter('ExperimentNumber', 'RaNGe', [1 4]);

experiments = Filter.filter( experiments ); % apply filter

%% ************************************************************************
% Step:    
% setup estimation problem
% *************************************************************************

% initialize problem
ParameterEstimationProblem = falcon.parest.Problem('B777LatEstimation');
% add model, states, controls, parameters, outputs
ParameterEstimationProblem.addModel( @B777Lat, IdParameters, states, controls, outputs );

%% ************************************************************************
% Step:    
% add experiments to problem
% *************************************************************************

ParameterEstimationProblem.addNewExperiment( experiments );
% generate initial guess for states
experiments.makeStateInitialGuess('simulate'); 

for iExp = experiments(:).'
    z = iExp.getMeasurementValues();
    x = z{1};
    for iX = 1:size(z{1}, 1)
        x(iX, :) = smooth( x(iX, :), 20 )';
    end
    
    iExp.makeStateInitialGuess('set', x);
end

%% ************************************************************************
% Step:    
% add cost function to problem
% *************************************************************************

costfunction = falcon.parest.costfunction.MaximumLikelihood();
% costfunction = falcon.parest.costfunction.MaximumLikelihood_mod();
% costfunction = falcon.parest.costfunction.LeastSquares();
% costfunction.J_y.R = 100*diag([0.07; 0.03; 0.07; 0.03].^2);
% costfunction.J_y.R = costfunction.J_y.R( [outputs.isConsidered], [outputs.isConsidered] );

% costfunction.ScalarScalingMode = 'auto_dJ_dy';  
costfunction.ScalarScalingMode = 'auto_J';  
% costfunction.ScalarScalingMode = 'off';  
% costfunction.ScalarScalingMode = 'manual';
% costfunction.J_y.ScalarScaling = 1/0.0015;
costfunction.J_y.CovarianceScalingMode = 'samples';  
% costfunction.J_y.CovarianceScalingMode = 'off';  
% costfunction.J_y.CovarianceScalingMode = 'manual';  
% costfunction.J_y.CovarianceScalingMatrix = diag([...
%     sqrt(2); 
%     sqrt(1); 
%     sqrt(1); 
%     ]);

costfunction.UpdatePlots = 4;   % update plots every x iterations

ParameterEstimationProblem.setCostFunction( costfunction );

%% ************************************************************************
% Step:    
% setup plots
% *************************************************************************

figure('units', 'normalized', 'outerposition', [0 0 1 1], ...
    'Name', 'Outputs B777')
% plot experiments
experiments.plotExperiment('Tags', {'Control', 'Type', 'Iteration'});

% configure and plot initial guess 
for iExperiment = 1:length(experiments)
    experiments(iExperiment).PlotCfg_model.Color = 'g';
end
experiments.plotModelOutput();

% configure and prepare progress plot
for iExperiment = 1:length(experiments)
    experiments(iExperiment).PlotCfg_model.Color = 'r';
end
experiments.plotModelOutput();

drawnow

%% ************************************************************************
% Step:    
% choose and configure solver and solve
% *************************************************************************

% ParameterEstimationProblem.improveInitialModelParameters('useInitialStateGuess', 'PrintLevel', 5); 
ParameterEstimationProblem.improveInitialModelParameters('EEM', 'PrintLevel', 5); 

% % ----------------- uncomment for SNOPT solution
% optimizer = falcon.solver.snopt();
% ParameterEstimationProblem.Bake()

% ----------------- uncomment for IPOPT solution
optimizer = falcon.solver.ipopt();
ParameterEstimationProblem.Bake()
% figure; ParameterEstimationProblem.CheckGradient()

% % % ----------------- uncomment for ML & Gauss-Newton solution
% % % OptimizationStepAlgorithm = falcon.parest.optimization.GaussNewton;
% OptimizationStepAlgorithm = falcon.parest.optimization.LevenbergMarquardt;
% optimizer = falcon.parest.solver( OptimizationStepAlgorithm );
% % optimizer.ParEstSolverOptions.ParallelWorkers = 3;
% optimizer.ParEstSolverOptions.ParallelProfile = 'local';

% set problem for solver
optimizer.setProblem(ParameterEstimationProblem);

% configure optimizer
optimizer.Options.MajorIterLimit = 300;
optimizer.Options.needH          = 0;

% solve problem
% figure; ParameterEstimationProblem.CheckGradient('visualize', true)
% figure; optimizer.checkDerivatives_J_p('visualize_dJ', true, ...
%     'visualize_ddJ', false)

optimizer.Solve();
%% ************************************************************************
% Step:    
% plot final results
% *************************************************************************

% compute parameter statistics
tic
ParameterEstimationProblem.computeParameterStatistics();
toc

% update model output plots
experiments.updateModelOutput();
experiments.plotUncertainties( 3 );

%%
% plot parameter statistics
figure('position', [20 700 1500 400])
subplot(1,2,1)
IdParameters.plotRelativeStandardDeviation();
subplot(1,2,2)
IdParameters.plotCorrelationMatrix();

% plot evolution of parameters
figure
IdParameters.plotParameterEvolution( 'SubplotConfig', [3, 5], 'Normalized', true);

%% ************************************************************************
% Step:    
% store resulting parameters to disc
% *************************************************************************

IdParameters.ToFile( fullfile( fileparts(which('data_B777Lat.mat')), 'parameters_B777Lat') )
