% script to generate example experimental data to test system
% identification solver - second order lag example

close all
clear
clc

%% ************************************************************************
% Step:    
% add necessary paths - use relative paths!
% *************************************************************************

falcon.init();

BasePath = fileparts(mfilename('fullpath'));
addpath([BasePath, filesep(), 'Model']);
% generate data directory if non-existent
if ~exist( fullfile(BasePath, 'Data'), 'dir' )
    mkdir(fullfile(BasePath, 'Data'));
end
addpath(fullfile(BasePath, 'Data'));

% setup model - i.e. build anew, if it has changed
buildPt2Model

%% ************************************************************************
% Step:    
% setup model parameters
% *************************************************************************

omega_0 = 10;
zeta = 0.2;
K = 4;
C = [0.5; 1];

p = [
    omega_0; 
    zeta; 
    K; 
    C;
    ];

%% ************************************************************************
% Step:    
% setup model
% *************************************************************************

% define measurement noise covariance matrix
NoiseCov = diag([0.5; 5].^2);
rng(42); % set random number generator to a defined state

% time step
delta_t = 0.01;

modelName  = 'Pt2';
modelFunc = str2func(modelName);
modelInfo  = modelFunc();

%% ************************************************************************
% Step:    
% perform experiments; for every experiment: 
%   1. increment index
%   2. create time vector
%   3. create inputs
%   4. use simulation model to generate clean data
%   5. add normally distributed, white noise
%   6. store data in experiment object
%   7. configure experiment tags
% *************************************************************************

iExp = 0;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 10;
t = 0:delta_t:t_end;

u = zeros(1, length(t));
u( t>2 & t<=4) = 1;
u( t>4 & t<6) = -1;

x0 = [8; 10;];

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).Tags.Name = 'experiment 1';
experiments(iExp).x0 = x0;

% ----------------------------------------------------------------------- 
iExp = iExp + 1;

t_end = 8;
t = 0:delta_t:t_end;

u = zeros(1, length(t));
u( t>2 & t<=3) = 1;
u( t>3 & t<4) = -1;

x0 = [4; -20;];

[~, y] = feval([func2str(modelFunc), '_sim'], t, x0, u, p);
z = y + NoiseCov.^0.5*randn(size(y));

experiments(iExp) = falcon.parest.Experiment(t, u, t, z);
experiments(iExp).Tags.Name = 'experiment 2';
experiments(iExp).x0 = x0;

%% ************************************************************************
% Step:    
% set measurement names
% *************************************************************************

experiments.setMeasurementNames( modelInfo.output(2).argnames );
experiments.setControlNames( modelInfo.input(2).argnames );

%% ************************************************************************
% Step:    
% set plot labels for measurements, controls, and time
% *************************************************************************

% --- example1: tex labels ---
% % experiments(1).Label_Cfg.Interpreter  = 'tex';
% % experiments(1).Label_Cfg.FontSize  = 17; 
% % experiments(1).Label_Cfg.FontName  = 'times new roman'; 
% 
% experiments.setMeasurementLabels( {'y_1', 'y_2'} );                                
% experiments.setControlLabels( {'x_{cmd}'} );

% --- example2: latex labels ---
experiments(1).Label_Cfg.Interpreter  = 'latex'; 
experiments(1).Label_Cfg.FontSize  = 17; 

experiments.setMeasurementLabels( {'$\mathit{y_1}$', '$\mathit{y_2}$'} );                                
experiments.setControlLabels( {'$\mathit{x_{cmd}}$'} );
experiments.setTimeLabel( '$\mathit{t}~[s]$' );

%% ************************************************************************
% Step:    
% plot experiment data
% *************************************************************************

figure('units', 'normalized', 'outerposition', [0 0 1 1])
experiments.plotExperiment()

%% ************************************************************************
% Step:    
% store data to disc
% *************************************************************************

experiments.ToFile(fullfile(BasePath, 'Data', 'data_Pt2.mat'))
